package com.mcwl.resource.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.mcwl.common.core.domain.entity.SysUser;
import com.mcwl.common.utils.SecurityUtils;
import com.mcwl.resource.domain.ModelComment;
import com.mcwl.resource.domain.ModelImageComment;
import com.mcwl.resource.domain.dto.ModelCommentRes;
import com.mcwl.resource.domain.dto.ModelImageCommentRes;
import com.mcwl.resource.domain.vo.ModelCommentVo;
import com.mcwl.resource.domain.vo.ModelImageCommentVo;
import com.mcwl.resource.mapper.ModelCommentMapper;
import com.mcwl.resource.mapper.ModelImageCommentMapper;
import com.mcwl.resource.mapper.ModelImageMapper;
import com.mcwl.resource.service.ModelCommentService;
import com.mcwl.system.service.ISysUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.validation.constraints.NotNull;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 模型评论
 */
@Service
public class ModelCommentServiceImpl extends ServiceImpl<ModelCommentMapper, ModelComment> implements ModelCommentService {

    @Autowired
    private  ISysUserService sysUserService;
    @Autowired
    private  ModelCommentMapper modelCommentMapper;

    @Autowired
    private ModelImageMapper modelImageMapper;

    @Override
    public void comment(ModelCommentRes modelCommentRes) {
        Long parentId = modelCommentRes.getParentId();

        if (Objects.nonNull(parentId)) {
            ModelComment mic = modelCommentMapper.selectById(parentId);
            if (Objects.isNull(mic)) {
                return;
            }
        }
        ModelComment modelComment = new ModelComment();
        BeanUtil.copyProperties(modelCommentRes, modelComment);
        modelComment.setUserId(SecurityUtils.getUserId());
        modelCommentMapper.insert(modelComment);

    }
    /**
     * 获取评论
     *
     * @param imageId 图片id
     * @return 评论区
     */
    @Override
    public List<ModelCommentVo> getComment(Long imageId) {

        // 1. 查询所有父评论
        List<ModelComment> parentComments = baseMapper.selectList(
                new LambdaQueryWrapper<ModelComment>()
                        .eq(ModelComment::getModelId, imageId)
                        .isNull(ModelComment::getParentId)
                        .orderByAsc(ModelComment::getCreateTime)
        );

        // 无评论直接返回空列表
        if (parentComments.isEmpty()) {
            return new ArrayList<>();
        }

        // 2. 收集所有用户ID（父评论+子评论）
        List<Long> userIds = collectUserIds(parentComments); // 收集父评论用户ID
        List<ModelComment> childComments = getChildComments(parentComments); // 查询子评论
        for (ModelComment child : childComments) {
            userIds.add(child.getUserId()); // 收集子评论用户ID
        }

        // 3. 批量查询用户信息（避免N+1查询）
        Map<Long, SysUser> userMap = userIds.isEmpty() ?
                new HashMap<>() :
                sysUserService.listByIds(userIds).stream()
                        .collect(Collectors.toMap(SysUser::getUserId, Function.identity()));

        // 4. 构建评论树结构
        return buildCommentTree(parentComments, childComments, userMap);
    }

    /**
     * 收集父评论的用户ID
     *
     * @param parentComments 父评论列表
     * @return 用户ID集合
     */
    private List<Long> collectUserIds(List<ModelComment> parentComments) {
        List<Long> userIds = new ArrayList<>();
        for (ModelComment comment : parentComments) {
            userIds.add(comment.getUserId());
        }
        return userIds;
    }

    /**
     * 批量查询子评论
     *
     * @param parentComments 父评论列表
     * @return 子评论列表
     */
    private List<ModelComment> getChildComments(List<ModelComment> parentComments) {
        if (parentComments.isEmpty()) {
            return new ArrayList<>();
        }
        // 提取父评论ID集合
        List<Long> parentIds = parentComments.stream()
                .map(ModelComment::getId)
                .collect(Collectors.toList());

        // 查询子评论
        return baseMapper.selectList(
                new LambdaQueryWrapper<ModelComment>()
                        .in(ModelComment::getParentId, parentIds)
                        .orderByAsc(ModelComment::getCreateTime)
        );
    }

    /**
     * 构建树形VO结构
     */
    private List<ModelCommentVo> buildCommentTree(List<ModelComment> parentComments,
                                                       List<ModelComment> childComments,
                                                       Map<Long, SysUser> userMap) {
        // 按父ID分组子评论
        Map<Long, List<ModelComment>> childMap = new HashMap<>();
        for (ModelComment child : childComments) {
            Long parentId = child.getParentId();
            childMap.computeIfAbsent(parentId, k -> new ArrayList<>()).add(child);
        }

        // 构建VO树
        List<ModelCommentVo> result = new ArrayList<>();
        for (ModelComment parent : parentComments) {
            // 转换父评论VO
            ModelCommentVo parentVo = convertToVo(parent, userMap);
            // 获取子评论VO列表
            List<ModelComment> children = childMap.getOrDefault(parent.getId(), new ArrayList<>());
            List<ModelCommentVo> childVos = new ArrayList<>();
            for (ModelComment child : children) {
                childVos.add(convertToVo(child, userMap));
            }
            parentVo.setContentList(childVos);
            result.add(parentVo);
        }
        return result;
    }

    /**
     * 转换单个评论为VO对象
     */
    private ModelCommentVo convertToVo(ModelComment comment, Map<Long, SysUser> userMap) {
        SysUser user = userMap.get(comment.getUserId());
        if (user == null) {
            user = createDefaultUser(); // 处理用户信息缺失
        }

        ModelCommentVo vo = new ModelCommentVo();
        vo.setUserId(comment.getUserId());
        vo.setUserName(user.getUserName());
        vo.setUserAvatar(user.getAvatar());
        vo.setCommentId(comment.getId());
        vo.setContent(comment.getContent());
        vo.setReplyUserId(comment.getReplyUserId());
        vo.setLikeNum(comment.getLikeNum());
        vo.setCreateTime(comment.getCreateTime());
        return vo;
    }

    /**
     * 创建默认用户（防止空指针）
     */
    private SysUser createDefaultUser() {
        SysUser defaultUser = new SysUser();
        defaultUser.setUserName("匿名用户");
        defaultUser.setAvatar("/default-avatar.png");
        return defaultUser;
    }


    /**
     * 构建ModelCommentVo对象
     *
     * @param modelComment 父评论对象
     * @return ModelCommentVo对象
     */
    @NotNull
    private ModelCommentVo getModelCommentVo(ModelComment modelComment) {
        Long userId = modelComment.getUserId();
        SysUser sysUser = sysUserService.selectUserById(userId);

        // 构建ModelCommentVo对象
        ModelCommentVo modelCommentVo = new ModelCommentVo();
        modelCommentVo.setUserId(userId);
        modelCommentVo.setUserName(sysUser.getUserName());
        modelCommentVo.setUserAvatar(sysUser.getAvatar());
        modelCommentVo.setCommentId(modelComment.getId());
        modelCommentVo.setContent(modelComment.getContent());
        // 查询子评论
        modelCommentVo.setContentList(getContentList(modelComment.getId()));
        modelCommentVo.setLikeNum(modelComment.getLikeNum());
        modelCommentVo.setCreateTime(modelComment.getCreateTime());
        return modelCommentVo;
    }


    /**
     * 递归查询子评论
     *
     * @param modelCommentId 父评论id
     * @return List<ModelCommentVo>
     */
    private List<ModelCommentVo> getContentList(Long modelCommentId) {
        List<ModelCommentVo> modelCommentVoList = new ArrayList<>();
        if (Objects.isNull(modelCommentId)) {
            return modelCommentVoList;
        }

        // 查询子评论
        LambdaQueryWrapper<ModelComment> lqw = new LambdaQueryWrapper<ModelComment>()
                .eq(ModelComment::getParentId, modelCommentId)
                .orderByDesc(ModelComment::getCreateTime);

        List<ModelComment> modelCommentList = modelCommentMapper.selectList(lqw);

        for (ModelComment modelComment : modelCommentList) {
            Long userId = modelComment.getUserId();
            SysUser sysUser = sysUserService.selectUserById(userId);
            ModelCommentVo modelCommentVo = new ModelCommentVo();
            modelCommentVo.setUserId(userId);
            modelCommentVo.setUserName(sysUser.getUserName());
            modelCommentVo.setUserAvatar(sysUser.getAvatar());
            modelCommentVo.setCommentId(modelComment.getId());
            modelCommentVo.setContent(modelComment.getContent());
            modelCommentVo.setLikeNum(modelComment.getLikeNum());
            modelCommentVo.setCreateTime(modelComment.getCreateTime());


            modelCommentVoList.add(modelCommentVo);
        }
        return modelCommentVoList;
    }


}
