package com.mcwl.resource.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.mcwl.common.core.domain.entity.SysUser;
import com.mcwl.common.utils.SecurityUtils;
import com.mcwl.resource.domain.ModelImage;
import com.mcwl.resource.domain.ModelImageComment;
import com.mcwl.resource.domain.dto.ModelImageCommentRes;
import com.mcwl.resource.domain.vo.ModelImageCommentVo;
import com.mcwl.resource.mapper.ModelImageCommentMapper;
import com.mcwl.resource.mapper.ModelImageMapper;
import com.mcwl.resource.service.ModelImageCommentService;
import com.mcwl.resource.service.ModelImageService;
import com.mcwl.system.service.ISysUserService;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;

import javax.validation.constraints.NotNull;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 图片评论实现
 */
@Service
@RequiredArgsConstructor
public class ModelImageCommentServiceImpl extends ServiceImpl<ModelImageCommentMapper, ModelImageComment> implements ModelImageCommentService {

    private final ISysUserService sysUserService;

    @Override
    public void comment(ModelImageCommentRes modelImageCommentRes) {
        Long parentId = modelImageCommentRes.getParentId();

        if (Objects.nonNull(parentId)) {
            ModelImageComment mic = baseMapper.selectById(parentId);
            if (Objects.isNull(mic)) {
                return;
            }
        }

        ModelImageComment modelImageComment = new ModelImageComment();
        BeanUtil.copyProperties(modelImageCommentRes, modelImageComment);
        modelImageComment.setUserId(SecurityUtils.getUserId());
        baseMapper.insert(modelImageComment);

    }

    /**
     * 获取评论
     *
     * @param imageId 图片id
     * @return 评论区
     */
    @Override
    public List<ModelImageCommentVo> getComment(Long imageId) {
        // 1. 查询所有父评论
        List<ModelImageComment> parentComments = baseMapper.selectList(
                new LambdaQueryWrapper<ModelImageComment>()
                        .eq(ModelImageComment::getModelImageId, imageId)
                        .isNull(ModelImageComment::getParentId)
                        .orderByAsc(ModelImageComment::getCreateTime)
        );

        // 无评论直接返回空列表
        if (parentComments.isEmpty()) {
            return new ArrayList<>();
        }

        // 2. 收集所有用户ID（父评论+子评论）
        List<Long> userIds = collectUserIds(parentComments); // 收集父评论用户ID
        List<ModelImageComment> childComments = getChildComments(parentComments); // 查询子评论
        for (ModelImageComment child : childComments) {
            userIds.add(child.getUserId()); // 收集子评论用户ID
        }

        // 3. 批量查询用户信息（避免N+1查询）
        Map<Long, SysUser> userMap = userIds.isEmpty() ?
                new HashMap<>() :
                sysUserService.listByIds(userIds).stream()
                        .collect(Collectors.toMap(SysUser::getUserId, Function.identity()));

        // 4. 构建评论树结构
        return buildCommentTree(parentComments, childComments, userMap);
    }

    /**
     * 收集父评论的用户ID
     *
     * @param parentComments 父评论列表
     * @return 用户ID集合
     */
    private List<Long> collectUserIds(List<ModelImageComment> parentComments) {
        List<Long> userIds = new ArrayList<>();
        for (ModelImageComment comment : parentComments) {
            userIds.add(comment.getUserId());
        }
        return userIds;
    }

    /**
     * 批量查询子评论
     *
     * @param parentComments 父评论列表
     * @return 子评论列表
     */
    private List<ModelImageComment> getChildComments(List<ModelImageComment> parentComments) {
        if (parentComments.isEmpty()) {
            return new ArrayList<>();
        }
        // 提取父评论ID集合
        List<Long> parentIds = parentComments.stream()
                .map(ModelImageComment::getId)
                .collect(Collectors.toList());

        // 查询子评论
        return baseMapper.selectList(
                new LambdaQueryWrapper<ModelImageComment>()
                        .in(ModelImageComment::getParentId, parentIds)
                        .orderByAsc(ModelImageComment::getCreateTime)
        );
    }

    /**
     * 构建树形VO结构
     */
    private List<ModelImageCommentVo> buildCommentTree(List<ModelImageComment> parentComments,
                                                       List<ModelImageComment> childComments,
                                                       Map<Long, SysUser> userMap) {
        // 按父ID分组子评论
        Map<Long, List<ModelImageComment>> childMap = new HashMap<>();
        for (ModelImageComment child : childComments) {
            Long parentId = child.getParentId();
            childMap.computeIfAbsent(parentId, k -> new ArrayList<>()).add(child);
        }

        // 构建VO树
        List<ModelImageCommentVo> result = new ArrayList<>();
        for (ModelImageComment parent : parentComments) {
            // 转换父评论VO
            ModelImageCommentVo parentVo = convertToVo(parent, userMap);
            // 获取子评论VO列表
            List<ModelImageComment> children = childMap.getOrDefault(parent.getId(), new ArrayList<>());
            List<ModelImageCommentVo> childVos = new ArrayList<>();
            for (ModelImageComment child : children) {
                childVos.add(convertToVo(child, userMap));
            }
            parentVo.setContentList(childVos);
            result.add(parentVo);
        }
        return result;
    }

    /**
     * 转换单个评论为VO对象
     */
    private ModelImageCommentVo convertToVo(ModelImageComment comment, Map<Long, SysUser> userMap) {
        SysUser user = userMap.get(comment.getUserId());
        if (user == null) {
            user = createDefaultUser(); // 处理用户信息缺失
        }

        ModelImageCommentVo vo = new ModelImageCommentVo();
        vo.setUserId(comment.getUserId());
        vo.setUserName(user.getUserName());
        vo.setUserAvatar(user.getAvatar());
        vo.setCommentId(comment.getId());
        vo.setContent(comment.getContent());
        vo.setReplyUserId(comment.getReplyUserId());
        vo.setLikeNum(comment.getLikeNum());
        vo.setCreateTime(comment.getCreateTime());
        return vo;
    }

    /**
     * 创建默认用户（防止空指针）
     */
    private SysUser createDefaultUser() {
        SysUser defaultUser = new SysUser();
        defaultUser.setUserName("匿名用户");
        defaultUser.setAvatar("/default-avatar.png");
        return defaultUser;
    }


}
