package com.mcwl.resource.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.mcwl.common.core.domain.entity.SysUser;
import com.mcwl.common.utils.SecurityUtils;
import com.mcwl.resource.domain.ModelImageComment;
import com.mcwl.resource.domain.WorkFlowComment;
import com.mcwl.resource.domain.dto.WorkFlowCommentRes;
import com.mcwl.resource.domain.vo.ModelImageCommentVo;
import com.mcwl.resource.domain.vo.WorkFlowCommentVo;
import com.mcwl.resource.mapper.WorkFlowCommentMapper;
import com.mcwl.resource.service.WorkFlowCommentService;
import com.mcwl.system.service.ISysUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.validation.constraints.NotNull;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 工作流评论 实现层
 *
 * @Author：ChenYan
 * @Project：McWl
 * @Package：com.mcwl.resource.service.impl
 * @Filename：ModelCommentServiceImpl
 * @Description TODO
 * @Date：2025/1/12 12:03
 */
@Service
public class WorkFlowCommentServiceImpl extends ServiceImpl<WorkFlowCommentMapper, WorkFlowComment> implements WorkFlowCommentService {

    @Autowired
    private ISysUserService sysUserService;

    @Autowired
    private WorkFlowCommentMapper workFlowCommentMapper;


    @Override
    public void comment(WorkFlowCommentRes workFlowCommentRes) {
        Long parentId = workFlowCommentRes.getParentId();
        if (parentId != null) {
            WorkFlowComment mic = workFlowCommentMapper.selectById(parentId);

            if (Objects.isNull(mic)) {
                return;
            }
        }
        WorkFlowComment workFlowComment = new WorkFlowComment();
        BeanUtil.copyProperties(workFlowCommentRes, workFlowComment);
        workFlowComment.setUserId(SecurityUtils.getUserId());
        workFlowCommentMapper.insert(workFlowComment);

    }

    @Override
    public List<WorkFlowCommentVo> getComment(Long imageId) {

        // 查询所有父评论
        List<WorkFlowComment> parentComments = baseMapper.selectList(
                new LambdaQueryWrapper<WorkFlowComment>()
                        .eq(WorkFlowComment::getWorkFlowId, imageId)
                        .isNull(WorkFlowComment::getParentId)
                        .orderByAsc(WorkFlowComment::getCreateTime)
        );

        // 无评论直接返回空列表
        if (parentComments.isEmpty()) {
            return new ArrayList<>();
        }

        // 2. 收集所有用户ID（父评论+子评论）
        List<Long> userIds = collectUserIds(parentComments); // 收集父评论用户ID
        List<WorkFlowComment> childComments = getChildComments(parentComments); // 查询子评论
        for (WorkFlowComment child : childComments) {
            userIds.add(child.getUserId()); // 收集子评论用户ID
        }

        // 3. 批量查询用户信息（避免N+1查询）
        Map<Long, SysUser> userMap = userIds.isEmpty() ?
                new HashMap<>() :
                sysUserService.listByIds(userIds).stream()
                        .collect(Collectors.toMap(SysUser::getUserId, Function.identity()));


        // 4. 构建评论树结构
        return buildCommentTree(parentComments, childComments, userMap);
    }

    /**
     * 收集父评论的用户ID
     *
     * @param parentComments 父评论列表
     * @return 用户ID集合
     */
    private List<Long> collectUserIds(List<WorkFlowComment> parentComments) {
        List<Long> userIds = new ArrayList<>();
        for (WorkFlowComment comment : parentComments) {
            userIds.add(comment.getUserId());
        }
        return userIds;
    }

    /**
     * 批量查询子评论
     *
     * @param parentComments 父评论列表
     * @return 子评论列表
     */
    private List<WorkFlowComment> getChildComments(List<WorkFlowComment> parentComments) {
        if (parentComments.isEmpty()) {
            return new ArrayList<>();
        }
        // 提取父评论ID集合
        List<Long> parentIds = parentComments.stream()
                .map(WorkFlowComment::getId)
                .collect(Collectors.toList());

        // 查询子评论
        return baseMapper.selectList(
                new LambdaQueryWrapper<WorkFlowComment>()
                        .in(WorkFlowComment::getParentId, parentIds)
                        .orderByAsc(WorkFlowComment::getCreateTime)
        );
    }

    /**
     * 构建树形VO结构
     */
    private List<WorkFlowCommentVo> buildCommentTree(List<WorkFlowComment> parentComments,
                                                       List<WorkFlowComment> childComments,
                                                       Map<Long, SysUser> userMap) {
        // 按父ID分组子评论
        Map<Long, List<WorkFlowComment>> childMap = new HashMap<>();
        for (WorkFlowComment child : childComments) {
            Long parentId = child.getParentId();
            childMap.computeIfAbsent(parentId, k -> new ArrayList<>()).add(child);
        }

        // 构建VO树
        List<WorkFlowCommentVo> result = new ArrayList<>();
        for (WorkFlowComment parent : parentComments) {
            // 转换父评论VO
            WorkFlowCommentVo parentVo = convertToVo(parent, userMap);
            // 获取子评论VO列表
            List<WorkFlowComment> children = childMap.getOrDefault(parent.getId(), new ArrayList<>());
            List<WorkFlowCommentVo> childVos = new ArrayList<>();
            for (WorkFlowComment child : children) {
                childVos.add(convertToVo(child, userMap));
            }
            parentVo.setContentList(childVos);
            result.add(parentVo);
        }
        return result;
    }

    /**
     * 转换单个评论为VO对象
     */
    private WorkFlowCommentVo convertToVo(WorkFlowComment comment, Map<Long, SysUser> userMap) {
        SysUser user = userMap.get(comment.getUserId());
        if (user == null) {
            user = createDefaultUser(); // 处理用户信息缺失
        }

        WorkFlowCommentVo vo = new WorkFlowCommentVo();
        vo.setUserId(comment.getUserId());
        vo.setUserName(user.getUserName());
        vo.setUserAvatar(user.getAvatar());
        vo.setCommentId(comment.getId());
        vo.setContent(comment.getContent());
        vo.setReplyUserId(comment.getReplyUserId());
        vo.setLikeNum(comment.getLikeNum());
        vo.setCreateTime(comment.getCreateTime());
        return vo;
    }

    /**
     * 创建默认用户（防止空指针）
     */
    private SysUser createDefaultUser() {
        SysUser defaultUser = new SysUser();
        defaultUser.setUserName("匿名用户");
        defaultUser.setAvatar("/default-avatar.png");
        return defaultUser;
    }



}
